// This file is part of noxos and licensed under the MIT open source license

#include <boot/config.h>
#include <drivers/builtin/fs/vfs.h>
#include "drivers/builtin/json.h"
#include <utils/logger.h>
#include <utils/memory.h>

sysconfig_T  G_sysconfig;
sysconfig_T* g_sysconfig = &G_sysconfig;

void sysconfig_init() {
    g_sysconfig->ps2_acpi_validation = true;
    g_sysconfig->log_graphical       = false;

    vfs_node_T* node = vfs_resolve_path(&g_root_fs, "/initrd/noxos.json");
    if (node == NULL || node->type != VFS_NODE_FILE) {
        log(LOG_WARNING, "No system config file found, falling back to standard values");
        return;
    }

    char* buffer = memory_allocate(node->size + 1);
    vfs_file_read(node, 0, node->size, buffer);
    buffer[node->size] = 0;

    json_T* json = json_from_string(buffer);

    json_node_T* child = json->root_node->childs_start;
    while (child != NULL) {
        if (string_compare("PS2_ACPI_VALIDATION", child->string)) {
            if (&child->childs_start[0] == NULL || child->childs_start[0].type != JSON_NODE_BOOL) {
                log(LOG_WARNING, "sysconfig -> failed to parse field 'PS2_ACPI_VALIDATION' (expected boolean)");
            } else {
                g_sysconfig->ps2_acpi_validation = child->childs_start[0].value;
            }
        } else if (string_compare("LOG_GRAPHICAL", child->string)) {
            if (&child->childs_start[0] == NULL || child->childs_start[0].type != JSON_NODE_BOOL) {
                log(LOG_WARNING, "sysconfig -> failed to parse field 'LOG_GRAPHICAL' (expected boolean)");
            } else {
                g_sysconfig->log_graphical = child->childs_start[0].value;
            }
        } else {
            log(LOG_WARNING, "sysconfig -> skipping field '%s' (unknown identifier)", child->string);
        }

        child = child->next;
    }

    json_destruct(json);

    memory_free(buffer);
}