// This file is part of noxos and licensed under the MIT open source license

#include "boot/boot_info.h"
#include "utils/logger.h"
#include "utils/core.h"
#include "utils/memory.h"

extern void kmain(boot_info_T boot_info);

static volatile struct limine_terminal_request terminal_request = {
        .id       = LIMINE_TERMINAL_REQUEST,
        .revision = 0
};

static volatile struct limine_framebuffer_request framebuffer_request = {
        .id       = LIMINE_FRAMEBUFFER_REQUEST,
        .revision = 1
};

static volatile struct limine_memmap_request memmap_request = {
        .id       = LIMINE_MEMMAP_REQUEST,
        .revision = 2
};

static volatile struct limine_kernel_file_request kernel_file_request = {
        .id       = LIMINE_KERNEL_FILE_REQUEST,
        .revision = 3
};

static volatile struct limine_module_request module_request = {
        .id       = LIMINE_MODULE_REQUEST,
        .revision = 4
};

static volatile struct limine_rsdp_request rsdp_request = {
        .id       = LIMINE_RSDP_REQUEST,
        .revision = 5
};


void _start() {
    boot_info_T boot_info;

    if (terminal_request.response == NULL || terminal_request.response->terminal_count < 0) {
        log(LOG_ERROR, "( LimineEntry ) no terminal response!");
        CORE_HALT_FOREVER
    }
    log(LOG_INFO, "( LimineEntry ) Found Terminal");
    boot_info.terminal = terminal_request.response;

    if (framebuffer_request.response == NULL || framebuffer_request.response->framebuffer_count < 0) {
        log(LOG_ERROR, "( LimineEntry ) no framebuffer response!");
        CORE_HALT_FOREVER
    }
    log(LOG_INFO, "( LimineEntry ) Found Framebuffer");
    boot_info.framebuffer = framebuffer_request.response;

    if (memmap_request.response == NULL || memmap_request.response->entry_count < 0) {
        log(LOG_ERROR, "( LimineEntry ) no memory map response!");
        CORE_HALT_FOREVER
    }
    log(LOG_INFO, "( LimineEntry ) Found Memory Map");
    boot_info.memory_map = memmap_request.response;

    if (kernel_file_request.response == NULL) {
        log(LOG_ERROR, "( LimineEntry ) no kernel file response!");
        CORE_HALT_FOREVER
    }
    log(LOG_INFO, "( LimineEntry ) Found Kernel File");
    boot_info.kernel_file = kernel_file_request.response->kernel_file;

    if (module_request.response == NULL) {
        log(LOG_ERROR, "( LimineEntry ) no module response!");
        CORE_HALT_FOREVER
    }
    struct limine_file* ramdisk_file = NULL;
    for (int i = 0; i < module_request.response->module_count; i++) {
        struct limine_file* file = module_request.response->modules[i];
        log(LOG_INFO, "( LimineEntry ) MODULE: %s", file->path);

        if (memory_compare(file->path, "/initrd.tar", string_length(file->path))) {
            ramdisk_file = file;
        }
    }

    if (ramdisk_file == NULL) {
        log(LOG_ERROR, "( LimineEntry ) no ramdisk module!");
        CORE_HALT_FOREVER
    }
    log(LOG_INFO, "( LimineEntry ) Found Ramdisk File");
    boot_info.ramdisk_file = ramdisk_file;

    if (rsdp_request.response == NULL) {
        log(LOG_ERROR, "( LimineEntry ) no RSDP response!");
        CORE_HALT_FOREVER
    }
    log(LOG_INFO, "( LimineEntry ) Found RSDP");
    boot_info.rsdp = rsdp_request.response->address;

    kmain(boot_info);
}