// This file is part of noxos and licensed under the MIT open source license

#include "drivers/builtin/elf/mapping.h"
#include "mm/page_frame.h"
#include "utils/math.h"
#include "utils/logger.h"
#include "utils/memory.h"

void elf_mapping_apply(elf_mapping_T* mapping, uint8_t* buffer, void* base, page_map_T* page_map) {
    uint64_t num_pages = CEIL_TO(mapping->length_virtual + (mapping->offset_virtual % PFRAME_SIZE), PFRAME_SIZE) / PFRAME_SIZE;
    log(LOG_INFO, "Mapping: 0x%x -> 0x%x  PAGES[%d]", mapping->offset_file, base + mapping->offset_virtual, num_pages);

    void* virtual_address = base + mapping->offset_virtual;

    for (uint64_t i = 0; i < num_pages; i++) {
        void* physical_address = pframe_request();
        page_map_map_memory(page_map, virtual_address + (PFRAME_SIZE * i), physical_address, PM_FLAG_READ_WRITE);
        page_map_map_memory(g_kernel_page_map, virtual_address + (PFRAME_SIZE * i), physical_address, PM_FLAG_READ_WRITE);
        log(LOG_INFO, "    mapped 0x%x", virtual_address + (PFRAME_SIZE * i));
    }

    if (mapping->length_virtual > mapping->length_file) {
        log(LOG_INFO, "    zeroing out area: 0x%x  <-->  0x%x", virtual_address + mapping->length_file, virtual_address + mapping->length_virtual);
        memory_set(virtual_address + mapping->length_file, 0, mapping->length_virtual - mapping->length_file-1);
    }

    log(LOG_INFO, "Copying data  0x%x --[%d]-> 0x%x", &buffer[mapping->offset_file], mapping->length_file, virtual_address);

    memory_copy(&buffer[mapping->offset_file], virtual_address, mapping->length_file);

    if (page_map != g_kernel_page_map) {
        for (uint64_t i = 0; i < num_pages; i++) {
            page_map_unmap_memory(g_kernel_page_map, virtual_address + (PFRAME_SIZE * i));
        }
    }
}

void elf_mappings_apply(elf_mapping_T* mappings, uint64_t num_mappings, uint8_t* buffer, void* base, page_map_T* page_map) {
    for (uint64_t i = 0; i < num_mappings; i++) {
        elf_mapping_apply(&mappings[i], buffer, base, page_map);
    }
}