// This file is part of noxos and licensed under the MIT open source license

#include <drivers/builtin/elf/relocation.h>
#include <drivers/builtin/elf/dynamic.h>
#include <drivers/builtin/elf/symbol.h>
#include <utils/logger.h>
#include <utils/memory.h>
#include <proc/scheduler.h>

//void elf_relocate(elf_executable_T* executable, uint8_t* buffer, void* address) {
//    elf_section_T* sections        = (elf_section_T*)&buffer[executable->header.offset_section_header];
//    elf_section_T* section_dynamic = NULL;
//    for (uint32_t i = 0; i < executable->header.num_section_header_entries; i++) {
//        if (sections[i].type != ELF_SECTION_DYNAMIC) continue;
//        section_dynamic = &sections[i];
//        DEBUG("Found DYNAMIC section %d", i);
//        break;
//    }
//    if (section_dynamic == NULL) {
//        log(LOG_ERROR, "Failed to relocate ELF executable (no .dynamic section found)");
//        return;
//    }
//    uint32_t       num_dynamic_entries  = section_dynamic->length / section_dynamic->entry_size;
//    elf_dynamic_T*     dynamic_entries  = (elf_dynamic_T*)&buffer[section_dynamic->offset];
//    uint32_t       num_needed_libraries = 0;
//
//    for (int i = 0; i < num_dynamic_entries; i++) {
//        switch (dynamic_entries[i].tag) {
//            case ELF_DYNAMIC_TAG_NEEDED: {
//                num_needed_libraries++;
//                break;
//            }
//        }
//    }
//
//    log(LOG_ERROR, "Failed to relocate executable (not implemented)");
//
//}

bool elf_relocate_driver(elf_executable_T* executable, uint8_t* buffer, void* load_address) {
    elf_section_T* sections        = (elf_section_T*)&buffer[executable->header.offset_section_header];
    string_t       section_names   = (string_t)&buffer[sections[executable->header.string_section_index].offset];
    elf_section_T* section_dynamic = NULL;
    elf_section_T* section_dynstr  = NULL;
    elf_section_T* section_dynsym  = NULL;

    for (uint32_t i = 0; i < executable->header.num_section_header_entries; i++) {
        switch (sections[i].type) {
            case ELF_SECTION_DYNAMIC: {
                section_dynamic = &sections[i];
                break;
            }
            case ELF_SECTION_STRING_TABLE: {
                if (!string_compare(".dynstr", &section_names[sections[i].name_offset])) break;
                section_dynstr = &sections[i];
                break;
            }
            case ELF_SECTION_DYNAMIC_SYMBOL_TABLE: {
                section_dynsym = &sections[i];
                break;
            }
        }
    }
    if (section_dynamic == NULL) {
        log(LOG_ERROR, "Failed to relocate driver executable (no .dynamic section found)");
        return false;
    }
    if (section_dynstr == NULL) {
        log(LOG_ERROR, "Failed to relocate driver executable (no .dynstr section found)");
        return false;
    }
    if (section_dynsym == NULL) {
        log(LOG_ERROR, "Failed to relocate driver executable (no .dynsym section found)");
        return false;
    }

    for (uint32_t i = 0; i < executable->header.num_section_header_entries; i++) {
        if (sections[i].type != ELF_SECTION_RELOCATION_A) continue;

        process_T*      kernel_process  = scheduler_get_process(PROCESS_KERNEL);
        symbol_table_T* symbol_table    = &kernel_process->executable->symbol_table;
        uint32_t        num_relocations = sections[i].length / sections[i].entry_size;
        log(LOG_INFO, "Found RELA section with %d relocations", num_relocations);

        for (int j = 0; j < num_relocations; j++) {
            elf_relocation_a_T*   relocation        = &((elf_relocation_a_T*)&buffer[sections[i].offset])[j];
            uint64_t*             relocation_offset = (uint64_t*)&((uint8_t*)load_address)[relocation->offset];
            elf_relocation_type_E relocation_type   = ELF_RELOCATION_TYPE(relocation->info);
            elf_symbol_T*         drv_symbol        = &((elf_symbol_T*)&buffer[section_dynsym->offset])[ELF_RELOCATION_SYMBOL(relocation->info)];
            string_t              drv_symbol_name   = (string_t)&buffer[section_dynstr->offset + drv_symbol->name_offset];
            symbol_T*             k_symbol          = symbol_resolve_from_name(symbol_table, drv_symbol_name);
            if (k_symbol == NULL) {
                log(LOG_ERROR, "Failed to relocate driver (symbol '%s' not found in kernel)", drv_symbol_name);
                return false;
            }

            log(LOG_INFO, "relocating '%s(0x%x)' -> PLT_ENTRY[0x%x]", drv_symbol_name, k_symbol, relocation_offset);

            switch (relocation_type) {
                case ELF_REL_JUMP_SLOT: {
                    *relocation_offset = k_symbol->address;
                    break;
                }
            }
        }
    }

    return true;
}