// This file is part of noxos and licensed under the MIT open source license

#include "drivers/builtin/fs/ramfs.h"
#include "utils/memory.h"
#include "utils/math.h"

void ramfs_file_delete(vfs_node_T* node) {
    if (node->cache == NULL) { return; }

    vfs_node_cache_destruct(node->cache);
}

uint64_t ramfs_file_write(vfs_node_T* node, uint64_t size, uint8_t* buffer_in) {
    if (node->cache != NULL) {
        vfs_node_cache_destruct(node->cache);
    }

    vfs_node_cache_create(node, size);

    memory_copy(buffer_in, node->cache->buffer, size);

    return size;
}

uint64_t ramfs_file_read(vfs_node_T* node, uint64_t size, uint8_t* buffer_out) {
    if (node->cache == NULL) { return 0; }

    memory_copy(node->cache->buffer, buffer_out, MIN(size, node->cache->buffer_size));
    return size;
}