// This file is part of noxos and licensed under the MIT open source license

#include "drivers/builtin/tty.h"
#include "utils/memory.h"

// this global state tty is just a placeholder
tty_T* g_tty;

void tty_init() {
    g_tty                  = memory_allocate(sizeof(tty_T));
    g_tty->graphics_buffer = graphics_buffer_request(0, 0, graphics_renderer_get_width(), graphics_renderer_get_height(), GRAPHICS_BUFFER_STANDARD);
    g_tty->cursor_overlay  = graphics_buffer_request(0, 0, 8, 16, GRAPHICS_BUFFER_OVERLAY);
    g_tty->cursor.x        = 0;
    g_tty->cursor.y        = 0;
    g_tty->color           = g_color_palette[COLOR_PAL_GREY_LIGHT];
    g_tty->output          = NULL;
    pipe_init(&g_tty->input, NULL, tty_update);

    graphics_buffer_draw_char(g_tty->cursor_overlay, 0, 0, g_color_palette[COLOR_PAL_GREY_DARK], '_');
}

void tty_update() {
    char     buffer[256];
    uint32_t read_bytes = 255;
    while (read_bytes == 255) {
        read_bytes = pipe_read(&g_tty->input, buffer, 255);
        buffer[read_bytes] = '\0';
        g_tty->cursor = graphics_buffer_draw_string(g_tty->graphics_buffer, g_tty->cursor.x, g_tty->cursor.y, g_tty->color, buffer);
    }

    graphics_buffer_move(g_tty->cursor_overlay, g_tty->cursor.x, g_tty->cursor.y);

    graphics_renderer_update();
}

uint32_t tty_write(string_t string) {
    pipe_write(g_tty->output, string, string_length(string));

    g_tty->cursor = graphics_buffer_draw_string(g_tty->graphics_buffer, g_tty->cursor.x, g_tty->cursor.y, g_tty->color, string);
    graphics_buffer_move(g_tty->cursor_overlay, g_tty->cursor.x, g_tty->cursor.y);
    graphics_renderer_update();
}