// This file is part of noxos and licensed under the MIT open source license

#include "mm/stack.h"
#include "mm/region.h"
#include "utils/logger.h"
#include "proc/scheduler.h"

void stack_dump_call_info(uint64_t rip, symbol_T* symbol) {
    log(LOG_NONE, "    0x%x+%xw  ->  %s", symbol->address, rip - symbol->address, symbol->name);
}

void stack_trace_call_stack(uint64_t rbp) {
    uint64_t    rip     = ((uint64_t*)rbp)[1];
    process_T* process = rip >= MEM_REGION_KERNEL ? scheduler_get_process(PROCESS_KERNEL) : scheduler_get_current_process();
    if (process == NULL) {
        log(LOG_NONE, "    0x%x       ->  <failed to resolve symbol>  (process NULL reference)", rip);
        return;
    }
    symbol_T*   symbol  = symbol_resolve_from_rip(&process->executable->symbol_table, rip);

    if (symbol == NULL) {
        log(LOG_NONE, "    0x%x       ->  <failed to resolve symbol>  (symbol NULL reference)", rip);
        return;
    }

    if (!string_compare(symbol->name, "_start")) {
        stack_trace_call_stack(((uint64_t*)rbp)[0]);
    }

    stack_dump_call_info(rip, symbol);
}