; This file is part of noxos and licensed under the MIT open source license

extern interrupts_handle
extern g_kernel_page_map

idt_load:
    lidt [rdi]
    ret
GLOBAL idt_load

interrupt_common_handler:
    ; save CPU state
    push    rbp
    push    rdi
    push    rsi
    push    rdx
    push    rcx
    push    rbx
    push    rax

    ; save page map
    mov     rax, cr3
    push    rax

    ; load kernel page map
    mov     rax, [g_kernel_page_map]
    mov     cr3, rax

    ; call handler
    mov     rdi, rsp
    call    interrupts_handle
    mov     rsp, rax

    ; load page map
    pop     rax
    mov     cr3, rax

    ; restore CPU state
    pop     rax
    pop     rbx
    pop     rcx
    pop     rdx
    pop     rsi
    pop     rdi
    pop     rbp

    ; remove error_code and interrupt number from stack
    add     rsp, 16

    iretq
GLOBAL interrupt_common_handler

%macro interrupt_stub 1
    interrupt_stub_%1:
        push qword    0
        push qword   %1
        jmp interrupt_common_handler
    GLOBAL interrupt_stub_%1
    %endmacro

%macro interrupt_stub_error_code 1
    interrupt_stub_%1:
        push qword   %1
        jmp interrupt_common_handler
    GLOBAL interrupt_stub_%1
    %endmacro

; Exceptions
interrupt_stub              0x00
interrupt_stub              0x01
interrupt_stub              0x02
interrupt_stub              0x03
interrupt_stub              0x04
interrupt_stub              0x05
interrupt_stub              0x06
interrupt_stub              0x07
interrupt_stub_error_code   0x08
interrupt_stub              0x09
interrupt_stub_error_code   0x0A
interrupt_stub_error_code   0x0B
interrupt_stub_error_code   0x0C
interrupt_stub_error_code   0x0D
interrupt_stub_error_code   0x0E
interrupt_stub              0x0F
interrupt_stub              0x10
interrupt_stub_error_code   0x11
interrupt_stub              0x12
interrupt_stub              0x13
interrupt_stub              0x14
interrupt_stub              0x15
interrupt_stub              0x16
interrupt_stub              0x17
interrupt_stub              0x18
interrupt_stub              0x19
interrupt_stub              0x1A
interrupt_stub              0x1B
interrupt_stub              0x1C
interrupt_stub              0x1D
interrupt_stub              0x1E
interrupt_stub              0x1F

;IRQs
interrupt_stub              0x20    ; PIT
interrupt_stub              0x21    ; Keyboard
interrupt_stub              0x22    ; Cascade
interrupt_stub              0x23    ; COM2
interrupt_stub              0x24    ; COM1
interrupt_stub              0x25    ; LPT2
interrupt_stub              0x26    ; Floppy
interrupt_stub              0x27    ; Spurious

interrupt_stub              0x28    ; CMOS RTC
interrupt_stub              0x29    ; Free 0
interrupt_stub              0x2A    ; Free 1
interrupt_stub              0x2B    ; Free 2
interrupt_stub              0x2C    ; PS/2 Mouse
interrupt_stub              0x2D    ; FPU
interrupt_stub              0x2E    ; Primary ATA
interrupt_stub              0x2F    ; Secondary ATA

; Custom
interrupt_stub              0x80    ; Syscall