// This file is part of noxos and licensed under the MIT open source license

#include "utils/bitmap.h"
#include "utils/memory.h"

bitmap_T bitmap_init_from_buffer(void* buffer, uint32_t size) {
    bitmap_T bitmap;
    bitmap.buffer    = buffer;
    bitmap.size      = size;
    bitmap.size_bits = size * 8;
    return bitmap;
}

bitmap_T bitmap_init(uint32_t size) {
    bitmap_T bitmap;
    bitmap.size      = size / 8 + 1;
    bitmap.size_bits = size;
    bitmap.buffer    = memory_allocate(bitmap.size);
    memory_set(bitmap.buffer, 0, bitmap.size);
    return bitmap;
}

void bitmap_destruct(bitmap_T* bitmap) {
    memory_free(bitmap->buffer);
}

bool bitmap_set(bitmap_T* bitmap, uint32_t index, bool value) {
    if (index >= bitmap->size_bits) { return false; }

    uint32_t index_byte = index / 8;
    uint8_t  index_bit  = index % 8;

    bitmap->buffer[index_byte] &= ~(0b10000000 >> index_bit);

    if (value) {
        bitmap->buffer[index_byte] |= (0b10000000 >> index_bit);
    }

    return true;
}

bool bitmap_get(bitmap_T* bitmap, uint32_t index) {
    if (index >= bitmap->size_bits) { return false; }

    uint32_t index_byte = index / 8;
    uint8_t  index_bit  = index % 8;

    return (bitmap->buffer[index_byte] & (0b10000000 >> index_bit)) > 0;
}