// This file is part of noxos and licensed under the MIT open source license

#include "utils/logger.h"
#include "utils/io.h"
#include "utils/core.h"
#include "drivers/builtin/graphics/renderer.h"

string_t g_log_prefixes[LOG_ENUM_END] = {
        "", // LOG_NONE
        "[  Info   ] ",
        "[  Debug  ] ",
        "[ Warning ] ",
        "[  Error  ] ",
};

color_palette_E g_log_colors[LOG_ENUM_END] = {
        COLOR_PAL_GREY_LIGHT,
        COLOR_PAL_GREY_DARK,
        COLOR_PAL_PINK,
        COLOR_PAL_ORANGE,
        COLOR_PAL_RED
};

bool                g_logger_blocked         = false;
graphics_buffer_T*  g_graphical_log_buffer   = NULL;
position_T          g_graphical_log_position;

void log_string(string_t str, log_level_E log_level) {
    char* c = (char*)str;
    while (*c != '\0') {
        io_out_byte(LOG_PORT, *c);
        c++;
    }

    if (g_graphical_log_buffer != NULL) {
        g_graphical_log_position = graphics_buffer_draw_string(g_graphical_log_buffer, g_graphical_log_position.x, g_graphical_log_position.y, g_color_palette[g_log_colors[log_level]], str);
    }
}

void graphical_log_init() {
    g_graphical_log_buffer   = graphics_buffer_request(0, 0, graphics_renderer_get_width() / 2, graphics_renderer_get_height(), GRAPHICS_BUFFER_OVERLAY);
    g_graphical_log_position = (position_T){0, 0};

    log(LOG_INFO, "<Logger> Started graphical log");
}

void log(log_level_E log_level, string_t str, ...) {
//    CORE_HALT_WHILE(g_logger_blocked)
    g_logger_blocked = true;

    va_list args;
    va_start(args, str);

    uint64_t size = variadic_format_size(str, args);
    char formatted_string[size];

    va_start(args, str);
    variadic_format(formatted_string, str, args);

    va_end(args);

    log_string(g_log_prefixes[log_level], log_level);
    log_string(formatted_string, log_level);
    log_string("\n", log_level);

    if (g_graphical_log_buffer != NULL) {
        graphics_renderer_update();
    }

    g_logger_blocked = false;
}