// This file is part of noxos and licensed under the MIT open source license

#include "utils/symbol.h"
#include "utils/memory.h"

#include "utils/logger.h"

symbol_table_T symbol_table_init(uint64_t num_symbols) {
    symbol_table_T symbol_table;

    DEBUG("symbol_table_init -> num_symbols: %d", num_symbols);

    symbol_table.symbols       = memory_allocate(num_symbols * sizeof(symbol_T));
    symbol_table.hashmap       = hashmap_create(num_symbols);
    symbol_table.num_symbols   = num_symbols;
    symbol_table.symbols_index = 0;

    return symbol_table;
}

void symbol_table_destruct(symbol_table_T* symbol_table) {
    hashmap_destruct(&symbol_table->hashmap);
    memory_free(symbol_table->symbols);
}
void symbol_table_insert(symbol_table_T* symbol_table, symbol_T symbol) {
//    DEBUG("symbol_table_insert -> SYMTAB[0x%x]  ||  %s (0x%x)", symbol_table, symbol.name, symbol.address);
    symbol_table->symbols[symbol_table->symbols_index] = symbol;
    hashmap_insert(&symbol_table->hashmap, string_hash_djb(symbol.name), &symbol_table->symbols[symbol_table->symbols_index]);

    symbol_table->symbols_index++;
}

symbol_T* symbol_resolve_from_name(symbol_table_T* symbol_table, string_t name) {
    return hashmap_lookup(&symbol_table->hashmap, string_hash_djb(name));
}

symbol_T* symbol_resolve_from_rip(symbol_table_T* symbol_table, uint64_t rip) {
    symbol_T* symbol = NULL;

    for (uint64_t i = 0; i < symbol_table->num_symbols; i++) {
        if (symbol_table->symbols[i].address <= rip &&
            (symbol == NULL || symbol_table->symbols[i].address > symbol->address))
        {
            symbol = &symbol_table->symbols[i];
        }
    }

    return symbol;
}